
export type temporary = any | void;


//db - SQLitePlugin (Экземпляр openDatabase)
export interface SQLitePlugin {
  readonly dbname: string
  openError: (e: any) => temporary
  openSuccess: (e: any) => temporary
  readonly openargs: {
      name?: string,
      location?: string,
      androidDatabaseProvider?: string,
      dblocation?: string,
      androidOldDatabaseImplementation?: number
  }
  
  abortAllPendingTransactions?: () => temporary
  addTransaction: (t: any) => temporary
  close: (success?: successCheckedType, error?: errorCheckedType) => temporary
  open: (success?: (db: SQLitePlugin) => void | any, error?: (err: Error | any) => void | any) => temporary
  databaseFeatures?: {isSQLitePluginDatabase: boolean} 
  executeSql: (
    statement: string,//SQL строка
    params?: any[],
    success?: (res: IResourceSuccess) => temporary,
    error?: (err: IResourceError) => temporary
  ) => temporary
  openDBs: any//{'arm.db': "OPEN" | string}
  readTransaction: (
    fn: fnTransitionType,
    error: errorCheckedType,
    success: successCheckedType
  ) => temporary
  sqlBatch: (
    sqlStatements: any[],
    /*
      [
        'CREATE TABLE MyTable IF NOT EXISTS (name STRING, balance INTEGER)',
        [ 'INSERT INTO MyTable VALUES (?1,?2)', ['Alice', 100] ],
        [ 'INSERT INTO MyTable VALUES (?1,?2)', ['Betty', 200] ],
      ]
    */
    success: successCheckedType,
    error: errorCheckedType
  ) => temporary
  startNextTransaction: () => temporary
  transaction: (
    fn: fnTransitionType,
    error?: errorCheckedType,
    success?: successCheckedType
  ) => void
}

//tx
export interface SQLitePluginTransaction {
  db: SQLitePlugin
  error: any
  executes: []
  finalized: boolean
  fn: (tx, res)=> temporary
  readOnly: boolean
  success: any
  txlock: boolean

  abort: (txFailure: temporary) => temporary
  abortFromQ: (sqlerror: temporary) => temporary
  addStatement: (sql: string, values?: any[], success?:SuccessTxAndResource, error?: ErrorTxAndErr) => temporary
  executeSql: (sql: string, values?: any[], success?:SuccessTxAndResource, error?: ErrorTxAndErr) => temporary
  finish: () => temporary
  handleStatementFailure: (handler: temporary, response: temporary) => temporary
  handleStatementSuccess: (handler: temporary, response: temporary) => temporary
  run: () => temporary
  start: () => temporary
}

export interface IResourceSuccess {
  readonly insertId: number | undefined
  rows: {item: (i: number) => any, readonly length: number }
  readonly rowsAffected: number
}

export interface IResourceError {
  readonly code: number
  readonly message: string
}

type fnTransitionType = (tx: SQLitePluginTransaction) => temporary

type successCheckedType = () => temporary
type errorCheckedType = (err: Error ) => temporary

type SuccessTxAndResource = (tx: SQLitePluginTransaction, res: IResourceSuccess) => temporary
type ErrorTxAndErr = (tx: SQLitePluginTransaction, err: IResourceError) => temporary



export interface openDatabaseOptionType{
  name: string,
  location: 'default',
  androidDatabaseProvider?: 'system'
}


export interface sqlitePluginType {
  openDatabase: (param:openDatabaseOptionType, success?: (db: SQLitePlugin) => void, error?: errorCheckedType) => SQLitePlugin; 
  deleteDatabase: (first: {name: string, location: 'default'}, success: successCheckedType, error: errorCheckedType) => void
  echoTest: (okcb, errorcb) => any | void; 
  selfTest: (successcb, errorcb) => temporary 
  sqliteFeatures?: {isSQLitePlugin: boolean}
}


export declare global {
  interface Window {
    db: any;
    sqlitePlugin: sqlitePluginType
  }
}
